@@ -5,8 +5,11 @@
 
 #include "cmConfigure.h" // IWYU pragma: keep
 
+#include <utility>
 #include <vector>
 
+#include "cmListFileCache.h"
+
 /**
  * Graph edge representation.  Most use cases just need the
  * destination vertex, so we support conversion to/from an int.  We
@@ -15,18 +18,26 @@
 class cmGraphEdge
 {
 public:
-  cmGraphEdge(int n = 0, bool s = true)
+  cmGraphEdge(int n, bool s, bool c, cmListFileBacktrace bt)
     : Dest(n)
     , Strong(s)
+    , Cross(c)
+    , Backtrace(std::move(bt))
   {
   }
   operator int() const { return this->Dest; }
 
   bool IsStrong() const { return this->Strong; }
 
+  bool IsCross() const { return this->Cross; }
+
+  cmListFileBacktrace const& GetBacktrace() const { return this->Backtrace; }
+
 private:
   int Dest;
   bool Strong;
+  bool Cross;
+  cmListFileBacktrace Backtrace;
 };
 struct cmGraphEdgeList : public std::vector<cmGraphEdge>
 {
