@@ -5,6 +5,8 @@
 
 #include "cmConfigure.h" // IWYU pragma: keep
 
+#include <string>
+
 class cmProperty
 {
 public:
@@ -22,4 +24,6 @@ public:
   };
 };
 
+using cmProp = const std::string*;
+
 #endif
