@@ -10,6 +10,8 @@
 #include <utility>
 #include <vector>
 
+#include "cmProperty.h"
+
 /** \class cmPropertyMap
  * \brief String property map.
  */
@@ -27,11 +29,11 @@ public:
   void SetProperty(const std::string& name, const char* value);
 
   //! Append to the property value
-  void AppendProperty(const std::string& name, const char* value,
+  void AppendProperty(const std::string& name, const std::string& value,
                       bool asString = false);
 
   //! Get the property value
-  const char* GetPropertyValue(const std::string& name) const;
+  cmProp GetPropertyValue(const std::string& name) const;
 
   //! Remove the property @a name from the map
   void RemoveProperty(const std::string& name);
