@@ -4,16 +4,17 @@
 
 #include "cmConfigure.h" // IWYU pragma: keep
 
-#include "cm_jsoncpp_value.h"
-#include "cm_thread.hxx"
-#include "cm_uv.h"
-
-#include "cmUVHandlePtr.h"
-
-#include <memory> // IWYU pragma: keep
+#include <memory>
 #include <string>
 #include <vector>
 
+#include <cm/shared_mutex>
+
+#include <cm3p/json/value.h>
+#include <cm3p/uv.h>
+
+#include "cmUVHandlePtr.h"
+
 class cmConnection;
 class cmFileMonitor;
 class cmServerProtocol;
@@ -88,20 +89,21 @@ protected:
 
 class cmServer : public cmServerBase
 {
-  CM_DISABLE_COPY(cmServer)
-
 public:
   class DebugInfo;
 
   cmServer(cmConnection* conn, bool supportExperimental);
   ~cmServer() override;
 
+  cmServer(cmServer const&) = delete;
+  cmServer& operator=(cmServer const&) = delete;
+
   bool Serve(std::string* errorMessage) override;
 
   cmFileMonitor* FileMonitor() const;
 
 private:
-  void RegisterProtocol(cmServerProtocol* protocol);
+  void RegisterProtocol(std::unique_ptr<cmServerProtocol> protocol);
 
   // Callbacks from cmServerConnection:
 
@@ -118,9 +120,10 @@ public:
   void OnConnected(cmConnection* connection) override;
 
 private:
-  static void reportProgress(const char* msg, float progress, void* data);
-  static void reportMessage(const char* msg, const char* title, bool& cancel,
-                            void* data);
+  static void reportProgress(const std::string& msg, float progress,
+                             const cmServerRequest& request);
+  static void reportMessage(const std::string& msg, const char* title,
+                            const cmServerRequest& request);
 
   // Handle requests:
   cmServerResponse SetProtocolVersion(const cmServerRequest& request);
@@ -146,12 +149,13 @@ private:
                        const DebugInfo* debug) const;
 
   static cmServerProtocol* FindMatchingProtocol(
-    const std::vector<cmServerProtocol*>& protocols, int major, int minor);
+    const std::vector<std::unique_ptr<cmServerProtocol>>& protocols, int major,
+    int minor);
 
   const bool SupportExperimental;
 
   cmServerProtocol* Protocol = nullptr;
-  std::vector<cmServerProtocol*> SupportedProtocols;
+  std::vector<std::unique_ptr<cmServerProtocol>> SupportedProtocols;
 
   friend class cmServerProtocol;
   friend class cmServerRequest;
