@@ -4,6 +4,7 @@
 
 #include <set>
 #include <sstream>
+#include <unordered_set>
 
 #include "cmExecutionStatus.h"
 #include "cmGlobalGenerator.h"
@@ -43,7 +44,9 @@ bool HandleSourceMode(cmExecutionStatus& status,
                       const std::set<std::string>& names,
                       const std::string& propertyName,
                       const std::string& propertyValue, bool appendAsString,
-                      bool appendMode, bool remove);
+                      bool appendMode, bool remove,
+                      const std::vector<cmMakefile*>& directory_makefiles,
+                      bool source_file_paths_should_be_absolute);
 bool HandleSource(cmSourceFile* sf, const std::string& propertyName,
                   const std::string& propertyValue, bool appendAsString,
                   bool appendMode, bool remove);
@@ -74,6 +77,147 @@ bool HandleInstall(cmInstalledFile* file, cmMakefile& makefile,
                    bool appendMode, bool remove);
 }
 
+namespace SetPropertyCommand {
+bool HandleSourceFileDirectoryScopes(
+  cmExecutionStatus& status, std::vector<std::string>& source_file_directories,
+  std::vector<std::string>& source_file_target_directories,
+  std::vector<cmMakefile*>& directory_makefiles)
+{
+  std::unordered_set<cmMakefile*> directory_makefiles_set;
+
+  cmMakefile* current_dir_mf = &status.GetMakefile();
+  if (!source_file_directories.empty()) {
+    for (const std::string& dir_path : source_file_directories) {
+      const std::string absolute_dir_path = cmSystemTools::CollapseFullPath(
+        dir_path, current_dir_mf->GetCurrentSourceDirectory());
+      cmMakefile* dir_mf =
+        status.GetMakefile().GetGlobalGenerator()->FindMakefile(
+          absolute_dir_path);
+      if (!dir_mf) {
+        status.SetError(cmStrCat("given non-existent DIRECTORY ", dir_path));
+        return false;
+      }
+      if (directory_makefiles_set.find(dir_mf) ==
+          directory_makefiles_set.end()) {
+        directory_makefiles.push_back(dir_mf);
+        directory_makefiles_set.insert(dir_mf);
+      }
+    }
+  }
+
+  if (!source_file_target_directories.empty()) {
+    for (const std::string& target_name : source_file_target_directories) {
+      cmTarget* target = current_dir_mf->FindTargetToUse(target_name);
+      if (!target) {
+        status.SetError(cmStrCat(
+          "given non-existent target for TARGET_DIRECTORY ", target_name));
+        return false;
+      }
+      cmProp target_source_dir = target->GetProperty("SOURCE_DIR");
+      cmMakefile* target_dir_mf =
+        status.GetMakefile().GetGlobalGenerator()->FindMakefile(
+          *target_source_dir);
+
+      if (directory_makefiles_set.find(target_dir_mf) ==
+          directory_makefiles_set.end()) {
+        directory_makefiles.push_back(target_dir_mf);
+        directory_makefiles_set.insert(target_dir_mf);
+      }
+    }
+  }
+
+  if (source_file_directories.empty() &&
+      source_file_target_directories.empty()) {
+    directory_makefiles.push_back(current_dir_mf);
+  }
+  return true;
+}
+
+bool HandleSourceFileDirectoryScopeValidation(
+  cmExecutionStatus& status, bool source_file_directory_option_enabled,
+  bool source_file_target_option_enabled,
+  std::vector<std::string>& source_file_directories,
+  std::vector<std::string>& source_file_target_directories)
+{
+  // Validate source file directory scopes.
+  if (source_file_directory_option_enabled &&
+      source_file_directories.empty()) {
+    std::string errors = "called with incorrect number of arguments "
+                         "no value provided to the DIRECTORY option";
+    status.SetError(errors);
+    return false;
+  }
+  if (source_file_target_option_enabled &&
+      source_file_target_directories.empty()) {
+    std::string errors = "called with incorrect number of arguments "
+                         "no value provided to the TARGET_DIRECTORY option";
+    status.SetError(errors);
+    return false;
+  }
+  return true;
+}
+
+bool HandleAndValidateSourceFileDirectortoryScopes(
+  cmExecutionStatus& status, bool source_file_directory_option_enabled,
+  bool source_file_target_option_enabled,
+  std::vector<std::string>& source_file_directories,
+  std::vector<std::string>& source_file_target_directories,
+  std::vector<cmMakefile*>& source_file_directory_makefiles)
+{
+  bool scope_options_valid =
+    SetPropertyCommand::HandleSourceFileDirectoryScopeValidation(
+      status, source_file_directory_option_enabled,
+      source_file_target_option_enabled, source_file_directories,
+      source_file_target_directories);
+  if (!scope_options_valid) {
+    return false;
+  }
+
+  scope_options_valid = SetPropertyCommand::HandleSourceFileDirectoryScopes(
+    status, source_file_directories, source_file_target_directories,
+    source_file_directory_makefiles);
+  return scope_options_valid;
+}
+
+std::string MakeSourceFilePathAbsoluteIfNeeded(
+  cmExecutionStatus& status, const std::string& source_file_path,
+  const bool needed)
+{
+  if (!needed) {
+    return source_file_path;
+  }
+  const std::string absolute_file_path = cmSystemTools::CollapseFullPath(
+    source_file_path, status.GetMakefile().GetCurrentSourceDirectory());
+  return absolute_file_path;
+}
+
+void MakeSourceFilePathsAbsoluteIfNeeded(
+  cmExecutionStatus& status,
+  std::vector<std::string>& source_files_absolute_paths,
+  std::vector<std::string>::const_iterator files_it_begin,
+  std::vector<std::string>::const_iterator files_it_end, const bool needed)
+{
+
+  // Make the file paths absolute, so that relative source file paths are
+  // picked up relative to the command calling site, regardless of the
+  // directory scope.
+  std::vector<std::string>::difference_type num_files =
+    files_it_end - files_it_begin;
+  source_files_absolute_paths.reserve(num_files);
+
+  if (!needed) {
+    source_files_absolute_paths.assign(files_it_begin, files_it_end);
+    return;
+  }
+
+  for (; files_it_begin != files_it_end; ++files_it_begin) {
+    const std::string absolute_file_path =
+      MakeSourceFilePathAbsoluteIfNeeded(status, *files_it_begin, true);
+    source_files_absolute_paths.push_back(absolute_file_path);
+  }
+}
+}
+
 bool cmSetPropertyCommand(std::vector<std::string> const& args,
                           cmExecutionStatus& status)
 {
@@ -114,13 +258,20 @@ bool cmSetPropertyCommand(std::vector<std::string> const& args,
   std::string propertyName;
   std::string propertyValue;
 
+  std::vector<std::string> source_file_directories;
+  std::vector<std::string> source_file_target_directories;
+  bool source_file_directory_option_enabled = false;
+  bool source_file_target_option_enabled = false;
+
   // Parse the rest of the arguments up to the values.
   enum Doing
   {
     DoingNone,
     DoingNames,
     DoingProperty,
-    DoingValues
+    DoingValues,
+    DoingSourceDirectory,
+    DoingSourceTargetDirectory
   };
   Doing doing = DoingNames;
   const char* sep = "";
@@ -137,8 +288,20 @@ bool cmSetPropertyCommand(std::vector<std::string> const& args,
       appendMode = true;
       remove = false;
       appendAsString = true;
+    } else if (doing != DoingProperty && doing != DoingValues &&
+               scope == cmProperty::SOURCE_FILE && arg == "DIRECTORY") {
+      doing = DoingSourceDirectory;
+      source_file_directory_option_enabled = true;
+    } else if (doing != DoingProperty && doing != DoingValues &&
+               scope == cmProperty::SOURCE_FILE && arg == "TARGET_DIRECTORY") {
+      doing = DoingSourceTargetDirectory;
+      source_file_target_option_enabled = true;
     } else if (doing == DoingNames) {
       names.insert(arg);
+    } else if (doing == DoingSourceDirectory) {
+      source_file_directories.push_back(arg);
+    } else if (doing == DoingSourceTargetDirectory) {
+      source_file_target_directories.push_back(arg);
     } else if (doing == DoingProperty) {
       propertyName = arg;
       doing = DoingValues;
@@ -159,6 +322,18 @@ bool cmSetPropertyCommand(std::vector<std::string> const& args,
     return false;
   }
 
+  std::vector<cmMakefile*> source_file_directory_makefiles;
+  bool file_scopes_handled =
+    SetPropertyCommand::HandleAndValidateSourceFileDirectortoryScopes(
+      status, source_file_directory_option_enabled,
+      source_file_target_option_enabled, source_file_directories,
+      source_file_target_directories, source_file_directory_makefiles);
+  if (!file_scopes_handled) {
+    return false;
+  }
+  bool source_file_paths_should_be_absolute =
+    source_file_directory_option_enabled || source_file_target_option_enabled;
+
   // Dispatch property setting.
   switch (scope) {
     case cmProperty::GLOBAL:
@@ -172,7 +347,9 @@ bool cmSetPropertyCommand(std::vector<std::string> const& args,
                               appendAsString, appendMode, remove);
     case cmProperty::SOURCE_FILE:
       return HandleSourceMode(status, names, propertyName, propertyValue,
-                              appendAsString, appendMode, remove);
+                              appendAsString, appendMode, remove,
+                              source_file_directory_makefiles,
+                              source_file_paths_should_be_absolute);
     case cmProperty::TEST:
       return HandleTestMode(status, names, propertyName, propertyValue,
                             appendAsString, appendMode, remove);
@@ -194,9 +371,8 @@ namespace {
 bool HandleGlobalMode(cmExecutionStatus& status,
                       const std::set<std::string>& names,
                       const std::string& propertyName,
-                      const std::string& propertyValue,
-                      const bool appendAsString, const bool appendMode,
-                      const bool remove)
+                      const std::string& propertyValue, bool appendAsString,
+                      bool appendMode, bool remove)
 {
   if (!names.empty()) {
     status.SetError("given names for GLOBAL scope.");
@@ -205,14 +381,14 @@ bool HandleGlobalMode(cmExecutionStatus& status,
 
   // Set or append the property.
   cmake* cm = status.GetMakefile().GetCMakeInstance();
-  const char* value = propertyValue.c_str();
-  if (remove) {
-    value = nullptr;
-  }
   if (appendMode) {
-    cm->AppendProperty(propertyName, value ? value : "", appendAsString);
+    cm->AppendProperty(propertyName, propertyValue, appendAsString);
   } else {
-    cm->SetProperty(propertyName, value);
+    if (remove) {
+      cm->SetProperty(propertyName, nullptr);
+    } else {
+      cm->SetProperty(propertyName, propertyValue.c_str());
+    }
   }
 
   return true;
@@ -221,9 +397,8 @@ bool HandleGlobalMode(cmExecutionStatus& status,
 bool HandleDirectoryMode(cmExecutionStatus& status,
                          const std::set<std::string>& names,
                          const std::string& propertyName,
-                         const std::string& propertyValue,
-                         const bool appendAsString, const bool appendMode,
-                         const bool remove)
+                         const std::string& propertyValue, bool appendAsString,
+                         bool appendMode, bool remove)
 {
   if (names.size() > 1) {
     status.SetError("allows at most one name for DIRECTORY scope.");
@@ -237,14 +412,8 @@ bool HandleDirectoryMode(cmExecutionStatus& status,
   if (!names.empty()) {
     // Construct the directory name.  Interpret relative paths with
     // respect to the current directory.
-    std::string dir = *names.begin();
-    if (!cmSystemTools::FileIsFullPath(dir)) {
-      dir = cmStrCat(status.GetMakefile().GetCurrentSourceDirectory(), '/',
-                     *names.begin());
-    }
-
-    // The local generators are associated with collapsed paths.
-    dir = cmSystemTools::CollapseFullPath(dir);
+    std::string dir = cmSystemTools::CollapseFullPath(
+      *names.begin(), status.GetMakefile().GetCurrentSourceDirectory());
 
     mf = status.GetMakefile().GetGlobalGenerator()->FindMakefile(dir);
     if (!mf) {
@@ -258,14 +427,14 @@ bool HandleDirectoryMode(cmExecutionStatus& status,
   }
 
   // Set or append the property.
-  const char* value = propertyValue.c_str();
-  if (remove) {
-    value = nullptr;
-  }
   if (appendMode) {
-    mf->AppendProperty(propertyName, value ? value : "", appendAsString);
+    mf->AppendProperty(propertyName, propertyValue, appendAsString);
   } else {
-    mf->SetProperty(propertyName, value);
+    if (remove) {
+      mf->SetProperty(propertyName, nullptr);
+    } else {
+      mf->SetProperty(propertyName, propertyValue.c_str());
+    }
   }
 
   return true;
@@ -274,9 +443,8 @@ bool HandleDirectoryMode(cmExecutionStatus& status,
 bool HandleTargetMode(cmExecutionStatus& status,
                       const std::set<std::string>& names,
                       const std::string& propertyName,
-                      const std::string& propertyValue,
-                      const bool appendAsString, const bool appendMode,
-                      const bool remove)
+                      const std::string& propertyValue, bool appendAsString,
+                      bool appendMode, bool remove)
 {
   for (std::string const& name : names) {
     if (status.GetMakefile().IsAlias(name)) {
@@ -300,18 +468,18 @@ bool HandleTargetMode(cmExecutionStatus& status,
 
 bool HandleTarget(cmTarget* target, cmMakefile& makefile,
                   const std::string& propertyName,
-                  const std::string& propertyValue, const bool appendAsString,
-                  const bool appendMode, const bool remove)
+                  const std::string& propertyValue, bool appendAsString,
+                  bool appendMode, bool remove)
 {
   // Set or append the property.
-  const char* value = propertyValue.c_str();
-  if (remove) {
-    value = nullptr;
-  }
   if (appendMode) {
-    target->AppendProperty(propertyName, value, appendAsString);
+    target->AppendProperty(propertyName, propertyValue, appendAsString);
   } else {
-    target->SetProperty(propertyName, value);
+    if (remove) {
+      target->SetProperty(propertyName, nullptr);
+    } else {
+      target->SetProperty(propertyName, propertyValue);
+    }
   }
 
   // Check the resulting value.
@@ -323,49 +491,57 @@ bool HandleTarget(cmTarget* target, cmMakefile& makefile,
 bool HandleSourceMode(cmExecutionStatus& status,
                       const std::set<std::string>& names,
                       const std::string& propertyName,
-                      const std::string& propertyValue,
-                      const bool appendAsString, const bool appendMode,
-                      const bool remove)
+                      const std::string& propertyValue, bool appendAsString,
+                      bool appendMode, bool remove,
+                      const std::vector<cmMakefile*>& directory_makefiles,
+                      const bool source_file_paths_should_be_absolute)
 {
-  for (std::string const& name : names) {
-    // Get the source file.
-    if (cmSourceFile* sf = status.GetMakefile().GetOrCreateSource(name)) {
-      if (!HandleSource(sf, propertyName, propertyValue, appendAsString,
-                        appendMode, remove)) {
+  std::vector<std::string> files_absolute;
+  std::vector<std::string> unique_files(names.begin(), names.end());
+  SetPropertyCommand::MakeSourceFilePathsAbsoluteIfNeeded(
+    status, files_absolute, unique_files.begin(), unique_files.end(),
+    source_file_paths_should_be_absolute);
+
+  for (const auto mf : directory_makefiles) {
+    for (std::string const& name : files_absolute) {
+      // Get the source file.
+      if (cmSourceFile* sf = mf->GetOrCreateSource(name)) {
+        if (!HandleSource(sf, propertyName, propertyValue, appendAsString,
+                          appendMode, remove)) {
+          return false;
+        }
+      } else {
+        status.SetError(cmStrCat(
+          "given SOURCE name that could not be found or created: ", name));
         return false;
       }
-    } else {
-      status.SetError(cmStrCat(
-        "given SOURCE name that could not be found or created: ", name));
-      return false;
     }
   }
+
   return true;
 }
 
 bool HandleSource(cmSourceFile* sf, const std::string& propertyName,
-                  const std::string& propertyValue, const bool appendAsString,
-                  const bool appendMode, const bool remove)
+                  const std::string& propertyValue, bool appendAsString,
+                  bool appendMode, bool remove)
 {
   // Set or append the property.
-  const char* value = propertyValue.c_str();
-  if (remove) {
-    value = nullptr;
-  }
-
   if (appendMode) {
-    sf->AppendProperty(propertyName, value, appendAsString);
+    sf->AppendProperty(propertyName, propertyValue, appendAsString);
   } else {
-    sf->SetProperty(propertyName, value);
+    if (remove) {
+      sf->SetProperty(propertyName, nullptr);
+    } else {
+      sf->SetProperty(propertyName, propertyValue.c_str());
+    }
   }
   return true;
 }
 
 bool HandleTestMode(cmExecutionStatus& status, std::set<std::string>& names,
                     const std::string& propertyName,
-                    const std::string& propertyValue,
-                    const bool appendAsString, const bool appendMode,
-                    const bool remove)
+                    const std::string& propertyValue, bool appendAsString,
+                    bool appendMode, bool remove)
 {
   // Look for tests with all names given.
   std::set<std::string>::iterator next;
@@ -396,18 +572,18 @@ bool HandleTestMode(cmExecutionStatus& status, std::set<std::string>& names,
 }
 
 bool HandleTest(cmTest* test, const std::string& propertyName,
-                const std::string& propertyValue, const bool appendAsString,
-                const bool appendMode, const bool remove)
+                const std::string& propertyValue, bool appendAsString,
+                bool appendMode, bool remove)
 {
   // Set or append the property.
-  const char* value = propertyValue.c_str();
-  if (remove) {
-    value = nullptr;
-  }
   if (appendMode) {
-    test->AppendProperty(propertyName, value, appendAsString);
+    test->AppendProperty(propertyName, propertyValue, appendAsString);
   } else {
-    test->SetProperty(propertyName, value);
+    if (remove) {
+      test->SetProperty(propertyName, nullptr);
+    } else {
+      test->SetProperty(propertyName, propertyValue.c_str());
+    }
   }
 
   return true;
@@ -416,9 +592,8 @@ bool HandleTest(cmTest* test, const std::string& propertyName,
 bool HandleCacheMode(cmExecutionStatus& status,
                      const std::set<std::string>& names,
                      const std::string& propertyName,
-                     const std::string& propertyValue,
-                     const bool appendAsString, const bool appendMode,
-                     const bool remove)
+                     const std::string& propertyValue, bool appendAsString,
+                     bool appendMode, bool remove)
 {
   if (propertyName == "ADVANCED") {
     if (!remove && !cmIsOn(propertyValue) && !cmIsOff(propertyValue)) {
@@ -445,7 +620,7 @@ bool HandleCacheMode(cmExecutionStatus& status,
   for (std::string const& name : names) {
     // Get the source file.
     cmake* cm = status.GetMakefile().GetCMakeInstance();
-    const char* existingValue = cm->GetState()->GetCacheEntryValue(name);
+    cmProp existingValue = cm->GetState()->GetCacheEntryValue(name);
     if (existingValue) {
       if (!HandleCacheEntry(name, status.GetMakefile(), propertyName,
                             propertyValue, appendAsString, appendMode,
@@ -463,21 +638,19 @@ bool HandleCacheMode(cmExecutionStatus& status,
 
 bool HandleCacheEntry(std::string const& cacheKey, const cmMakefile& makefile,
                       const std::string& propertyName,
-                      const std::string& propertyValue,
-                      const bool appendAsString, const bool appendMode,
-                      const bool remove)
+                      const std::string& propertyValue, bool appendAsString,
+                      bool appendMode, bool remove)
 {
   // Set or append the property.
-  const char* value = propertyValue.c_str();
   cmState* state = makefile.GetState();
   if (remove) {
     state->RemoveCacheEntryProperty(cacheKey, propertyName);
   }
   if (appendMode) {
-    state->AppendCacheEntryProperty(cacheKey, propertyName, value,
+    state->AppendCacheEntryProperty(cacheKey, propertyName, propertyValue,
                                     appendAsString);
   } else {
-    state->SetCacheEntryProperty(cacheKey, propertyName, value);
+    state->SetCacheEntryProperty(cacheKey, propertyName, propertyValue);
   }
 
   return true;
@@ -486,9 +659,8 @@ bool HandleCacheEntry(std::string const& cacheKey, const cmMakefile& makefile,
 bool HandleInstallMode(cmExecutionStatus& status,
                        const std::set<std::string>& names,
                        const std::string& propertyName,
-                       const std::string& propertyValue,
-                       const bool appendAsString, const bool appendMode,
-                       const bool remove)
+                       const std::string& propertyValue, bool appendAsString,
+                       bool appendMode, bool remove)
 {
   cmake* cm = status.GetMakefile().GetCMakeInstance();
 
@@ -510,17 +682,17 @@ bool HandleInstallMode(cmExecutionStatus& status,
 
 bool HandleInstall(cmInstalledFile* file, cmMakefile& makefile,
                    const std::string& propertyName,
-                   const std::string& propertyValue, const bool appendAsString,
-                   const bool appendMode, const bool remove)
+                   const std::string& propertyValue, bool appendAsString,
+                   bool appendMode, bool remove)
 {
   // Set or append the property.
-  const char* value = propertyValue.c_str();
   if (remove) {
     file->RemoveProperty(propertyName);
   } else if (appendMode) {
-    file->AppendProperty(&makefile, propertyName, value, appendAsString);
+    file->AppendProperty(&makefile, propertyName, propertyValue,
+                         appendAsString);
   } else {
-    file->SetProperty(&makefile, propertyName, value);
+    file->SetProperty(&makefile, propertyName, propertyValue);
   }
   return true;
 }
