@@ -7,8 +7,7 @@
 
 #include <cm/memory>
 #include <cm/string_view>
-
-#include "cm_static_string_view.hxx"
+#include <cmext/string_view>
 
 #include "cmConditionEvaluator.h"
 #include "cmExecutionStatus.h"
