@@ -12,6 +12,8 @@
 #include <utility>
 #include <vector>
 
+#include <cmext/algorithm>
+
 class cmGeneratorTarget;
 
 class cmXCodeObject
@@ -82,13 +84,11 @@ public:
   void AddObject(cmXCodeObject* value) { this->List.push_back(value); }
   bool HasObject(cmXCodeObject* o) const
   {
-    return !(std::find(this->List.begin(), this->List.end(), o) ==
-             this->List.end());
+    return cm::contains(this->List, o);
   }
   void AddUniqueObject(cmXCodeObject* value)
   {
-    if (std::find(this->List.begin(), this->List.end(), value) ==
-        this->List.end()) {
+    if (!cm::contains(this->List, value)) {
       this->List.push_back(value);
     }
   }
@@ -109,8 +109,7 @@ public:
   bool HasComment() const { return (!this->Comment.empty()); }
   cmXCodeObject* GetObject(const char* name) const
   {
-    std::map<std::string, cmXCodeObject*>::const_iterator i =
-      this->ObjectAttributes.find(name);
+    auto const i = this->ObjectAttributes.find(name);
     if (i != this->ObjectAttributes.end()) {
       return i->second;
     }
