@@ -1,7 +1,8 @@
 #include <assert.h>
+// clang-format off
 #include <stdio.h>
-
 #include <jpeglib.h>
+// clang-format on
 
 int main()
 {
@@ -12,5 +13,5 @@ int main()
   cinfo.err = jpeg_std_error(&jerr);
   jpeg_create_decompress(&cinfo);
 
-  return 0;
+  return (JPEG_LIB_VERSION != CMAKE_EXPECTED_JPEG_VERSION);
 }
